'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chai = require('chai');

var _buildTree = require('./build-tree');

var buildTree = _interopRequireWildcard(_buildTree);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var fixtures = `${__dirname}/../../fixtures`;
var precinctFixtures = _path2.default.join(fixtures, 'precinct');

describe('buildTree', function () {
  describe('getDependencyTree', function () {
    var dependencyTreeParams = {
      baseDir: '.',
      consumerPath: __dirname,
      filePaths: [],
      bindingPrefix: '@bit',
      resolveModulesConfig: undefined
    };
    it('when no files are passed should return an empty tree', _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
      var results;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return buildTree.getDependencyTree(dependencyTreeParams);

            case 2:
              results = _context.sent;

              (0, _chai.expect)(results).to.deep.equal({ tree: {} });

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));
    it('when unsupported files are passed should return them with no dependencies', _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
      var results;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              dependencyTreeParams.filePaths = [`${fixtures}/unsupported-file.pdf`];
              _context2.next = 3;
              return buildTree.getDependencyTree(dependencyTreeParams);

            case 3:
              results = _context2.sent;

              (0, _chai.expect)(results.tree).to.deep.equal({ 'fixtures/unsupported-file.pdf': {} });

            case 5:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));
    it('when supported and unsupported files are passed should return them all', _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
      var results;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              dependencyTreeParams.filePaths = [`${fixtures}/unsupported-file.pdf`, `${precinctFixtures}/es6.js`];
              _context3.next = 3;
              return buildTree.getDependencyTree(dependencyTreeParams);

            case 3:
              results = _context3.sent;

              (0, _chai.expect)(results.tree).to.have.property('fixtures/unsupported-file.pdf');
              (0, _chai.expect)(results.tree).to.have.property('fixtures/precinct/es6.js');

            case 6:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    })));
    it('when a file has parsing error it should add the file to the tree with the error instance', _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
      var results, unParsedFile;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              dependencyTreeParams.filePaths = [`${precinctFixtures}/unparseable.js`];
              _context4.next = 3;
              return buildTree.getDependencyTree(dependencyTreeParams);

            case 3:
              results = _context4.sent;
              unParsedFile = 'fixtures/precinct/unparseable.js';

              (0, _chai.expect)(results.tree).to.have.property(unParsedFile);
              (0, _chai.expect)(results.tree[unParsedFile]).to.have.property('error');
              (0, _chai.expect)(results.tree[unParsedFile].error).to.be.instanceof(Error);

            case 8:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));
    describe('missing dependencies', function () {
      var results = void 0;
      var missingDepsFile = 'fixtures/missing-deps.js';
      before(_asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                dependencyTreeParams.filePaths = [`${fixtures}/missing-deps.js`];
                _context5.next = 3;
                return buildTree.getDependencyTree(dependencyTreeParams);

              case 3:
                results = _context5.sent;

                (0, _chai.expect)(results.tree).to.have.property(missingDepsFile);
                (0, _chai.expect)(results.tree[missingDepsFile]).to.have.property('missing');

              case 6:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, undefined);
      })));
      it('it should add the missing dependency to the missing section in the tree', _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee6() {
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                (0, _chai.expect)(results.tree[missingDepsFile].missing).to.have.property('files');
                (0, _chai.expect)(results.tree[missingDepsFile].missing.files[0]).to.equal('../non-exist-dep');

              case 2:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, undefined);
      })));
      it('it should add the missing package to the missing section in the tree', _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                (0, _chai.expect)(results.tree[missingDepsFile].missing).to.have.property('packages');
                (0, _chai.expect)(results.tree[missingDepsFile].missing.packages[0]).to.equal('non-exist-package');

              case 2:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, undefined);
      })));
    });
  });
});
//# sourceMappingURL=build-tree.spec.js.map