'use strict';

var _chai = require('chai');

var _pathMap = require('../../fixtures/path-map.json');

var _pathMap2 = _interopRequireDefault(_pathMap);

var _pathMap3 = require('./path-map');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('path-map', function () {
  describe('updatePathMapWithLinkFilesData', function () {
    it('should return an empty array for an empty pathMap', function () {
      var results = (0, _pathMap3.getPathMapWithLinkFilesData)([]);
      (0, _chai.expect)(results).to.have.lengthOf(0);
    });
    it('should throw TypeError for a non array input', function () {
      (0, _chai.expect)(function () {
        return (0, _pathMap3.getPathMapWithLinkFilesData)();
      }).to.throw(TypeError);
    });
    it('should return the same pathMap when there are no link-files involved', function () {
      var fixture = [{ file: 'bar/foo.js', dependencies: [] }];
      var results = (0, _pathMap3.getPathMapWithLinkFilesData)(fixture);
      (0, _chai.expect)(results).to.deep.equal(fixture);
    });
    describe('a file bar/foo require dependencies isString and isArray through two levels of link files', function () {
      var results = void 0;
      var barFooDependency = void 0;
      before(function () {
        results = (0, _pathMap3.getPathMapWithLinkFilesData)(_pathMap2.default);
        var barFoo = results.find(function (f) {
          return f.file === 'bar/foo.js';
        });
        barFooDependency = barFoo.dependencies[0];
      });
      it('should mark the dependency as a linkFile', function () {
        (0, _chai.expect)(barFooDependency).to.have.property('linkFile');
        (0, _chai.expect)(barFooDependency.linkFile).to.be.true;
      });
      it('should add a new attribute realDependencies to the dependency', function () {
        (0, _chai.expect)(barFooDependency).to.have.property('realDependencies');
      });
      it('realDependencies should include the final dependencies skipping multiple links in between', function () {
        (0, _chai.expect)(barFooDependency.realDependencies).to.have.lengthOf(2);
        // ensures that it skips both: utils/index.js and utils/is-string/index.js
        (0, _chai.expect)(barFooDependency.realDependencies[0].file).to.equal('utils/is-string/is-string.js');
        // ensures that it skips both: utils/index.js and utils/is-array/index.js
        (0, _chai.expect)(barFooDependency.realDependencies[1].file).to.equal('utils/is-array/is-array.js');
      });
      it('realDependencies should have the importSpecifiers.mainFile same as the original file', function () {
        var isStringSpecifier = barFooDependency.importSpecifiers.find(function (i) {
          return i.name === 'isString';
        });
        var realDepIsStringSpecifier = barFooDependency.realDependencies[0].importSpecifiers[0];
        (0, _chai.expect)(realDepIsStringSpecifier.mainFile.name).to.equal(isStringSpecifier.name);
        (0, _chai.expect)(realDepIsStringSpecifier.mainFile.isDefault).to.equal(isStringSpecifier.isDefault);
      });
      it('realDependencies should have the importSpecifiers.linkFile same as the last link file', function () {
        var lastLinkFile = 'utils/is-string/index.js';
        var lastLink = results.find(function (f) {
          return f.file === lastLinkFile;
        });
        var lastLinkSpecifier = lastLink.dependencies[0].importSpecifiers[0];
        var realDepIsStringSpecifier = barFooDependency.realDependencies[0].importSpecifiers[0];
        (0, _chai.expect)(realDepIsStringSpecifier.linkFile.name).to.equal(lastLinkSpecifier.name);
        (0, _chai.expect)(realDepIsStringSpecifier.linkFile.isDefault).to.equal(lastLinkSpecifier.isDefault);
      });
    });
  });
});
//# sourceMappingURL=path-map.spec.js.map